@@ -34,7 +34,7 @@
 #'  library(survival)
 #'  CreateTableOne2(vars = names(lung), strata = "sex", data = lung)
 #' @rdname CreateTableOne2
-#' @importFrom data.table data.table :=
+#' @importFrom data.table data.table := setkey
 #' @importFrom tableone CreateTableOne 
 #' @importFrom labelled var_label var_label<-
 #' @importFrom stats chisq.test fisher.test kruskal.test oneway.test
@@ -49,19 +49,12 @@ CreateTableOne2 <- function(data, strata, vars, factorVars, includeNA = F, test
                            showAllLevels = T, printToggle = F, quote = F, smd = F, Labels = F, exact = NULL, nonnormal = NULL, 
                            catDigits = 1, contDigits = 2, pDigits = 3, labeldata = NULL){
   
-  if (Labels & !is.null(labeldata)){
-    labelled::var_label(data) = sapply(names(data), function(v){as.character(labeldata[get("variable") == v, "var_label"][1])}, simplify = F)
-    vals.tb1 <- c(NA, unlist(sapply(vars, function(v){labeldata[get("variable") == v, "val_label"]})))
-  }
-  
-  factor_vars <- vars[sapply(vars, function(x){class(data[[x]]) %in% c("factor", "character")})]
+  setkey <- variable <- level <- . <- val_label <- NULL
   
   if (length(strata) != 1){
     stop("Please select only 1 strata")
   }
   
-  vars.fisher <- sapply(factor_vars, function(x){is(tryCatch(chisq.test(table(data[[strata]], data[[x]])),error=function(e) e, warning=function(w) w), "warning")})
-  vars.fisher <- factor_vars[unlist(vars.fisher)]
   
   res <- tableone::CreateTableOne(vars =vars, strata = strata, data = data, factorVars = factorVars, includeNA = includeNA, test = test, 
                        testApprox = testApprox, argsApprox = argsApprox,
@@ -69,6 +62,28 @@ CreateTableOne2 <- function(data, strata, vars, factorVars, includeNA = F, test
                        testNormal = testNormal, argsNormal = argsNormal,
                        testNonNormal = testNonNormal, argsNonNormal = argsNonNormal, smd = smd)
   
+  #factor_vars <- vars[sapply(vars, function(x){class(data[[x]]) %in% c("factor", "character")})]
+  factor_vars <- res[["MetaData"]][["varFactors"]]
+  
+  if (Labels & !is.null(labeldata)){
+    labelled::var_label(data) = sapply(names(data), function(v){as.character(labeldata[get("variable") == v, "var_label"][1])}, simplify = F)
+    #vals.tb1 <- c(NA, unlist(sapply(vars, function(v){labeldata[get("variable") == v, "val_label"]})))
+    data.table::setkey(labeldata, variable, level)
+    res0 <- tableone::CreateTableOne(vars =vars, data = data, factorVars = factorVars, includeNA = includeNA)
+    for (i in seq_along(res0$CatTable)){
+      for(j in factor_vars){
+        lvs <- res0$CatTable[[i]][[j]]$level
+        res0$CatTable[[i]][[j]]$level <- labeldata[.(j, lvs), val_label]
+      }
+    }
+    ptb1.rn <- rownames(print(res0, showAllLevels = showAllLevels, printToggle = printToggle, quote = quote, varLabels = Labels, nonnormal = nonnormal,
+                              catDigits = catDigits, contDigits = contDigits))
+    ptb1.rn <- gsub("(mean (SD))", "", ptb1.rn, fixed=T)
+  }
+  
+  vars.fisher <- sapply(factor_vars, function(x){is(tryCatch(chisq.test(table(data[[strata]], data[[x]])),error=function(e) e, warning=function(w) w), "warning")})
+  vars.fisher <- factor_vars[unlist(vars.fisher)]
+  
   if (is.null(exact) & length(vars.fisher) > 0){
     exact <- vars.fisher
   }
@@ -76,13 +91,24 @@ CreateTableOne2 <- function(data, strata, vars, factorVars, includeNA = F, test
   ptb1 <- print(res,
                showAllLevels = showAllLevels, printToggle = printToggle, quote = quote, smd = smd, varLabels = Labels, nonnormal = nonnormal, exact = exact,
                catDigits = catDigits, contDigits = contDigits, pDigits = pDigits)
-  rownames(ptb1) = gsub("(mean (SD))", "", rownames(ptb1), fixed=T)
+  
+  rownames(ptb1) <- gsub("(mean (SD))", "", rownames(ptb1), fixed=T)
+  if (Labels & !is.null(labeldata)){
+    rownames(ptb1) <- ptb1.rn
+    
+  }
+  
   #cap.tb1 = paste("Table 1: Stratified by ", strata, sep="")
   
   if (Labels & !is.null(labeldata)){
-    colname.group_var = unlist(labeldata[get("variable") == strata, "val_label"])
-    colnames(ptb1)[1:(length(colname.group_var)+1)] = unlist(c(labeldata[get("variable") == strata, "var_label"][1], colname.group_var))
-    ptb1[,1] = vals.tb1
+    colname.group_var <- unlist(labeldata[.(strata, names(res$CatTable)), val_label])
+    if(showAllLevels == T){
+      #colname.group_var <- unlist(labeldata[get("variable") == strata, "val_label"])
+      colnames(ptb1)[1:(length(colname.group_var)+1)] <- unlist(c(labeldata[get("variable") == strata, "var_label"][1], colname.group_var))
+    } else{
+      colnames(ptb1)[1:length(colname.group_var)] <- colname.group_var
+    }
+    #ptb1[,1] = vals.tb1
     #cap.tb1 = paste("Table 1: Stratified by ", labeldata[variable == strata, "var_label"][1], sep="")
     
   }
@@ -147,7 +173,7 @@ CreateTableOneJS <- function(vars, strata = NULL, strata2 = NULL, data, factorVa
                             showAllLevels = T, printToggle = F, quote = F, smd = F, Labels = F, exact = NULL, nonnormal = NULL, 
                             catDigits = 1, contDigits = 2, pDigits = 3, labeldata = NULL, psub = T){
   
-  variable <- val_label <- V1 <- V2 <- NULL
+  . <- level <- variable <- val_label <- V1 <- V2 <- NULL
   #if (Labels & !is.null(labeldata)){
   #  var_label(data) = sapply(names(data), function(v){as.character(labeldata[get("variable") == v, "var_label"][1])}, simplify = F)
   #  vals.tb1 = c(NA, unlist(sapply(vars, function(v){labeldata[get("variable") == v, "val_label"]})))
@@ -155,24 +181,35 @@ CreateTableOneJS <- function(vars, strata = NULL, strata2 = NULL, data, factorVa
   data <- data
   
   if (is.null(strata)){
-    if (Labels & !is.null(labeldata)){
-      labelled::var_label(data) <- sapply(names(data), function(v){as.character(labeldata[get("variable") == v, "var_label"][1])}, simplify = F)
-      vals.tb1 <- c(NA, unlist(sapply(vars, function(v){labeldata[get("variable") == v, "val_label"]})))
-    }
     
     res <- tableone::CreateTableOne(vars =vars, data = data, factorVars = factorVars, includeNA = includeNA, test = test, 
                          testApprox = testApprox, argsApprox = argsApprox,
                          testExact = testExact, argsExact = argsExact,
                          testNormal = testNormal, argsNormal = argsNormal,
                          testNonNormal = testNonNormal, argsNonNormal = argsNonNormal, smd = smd)
+    
+    factor_vars <- res[["MetaData"]][["varFactors"]]
+    
+    if (Labels & !is.null(labeldata)){
+      labelled::var_label(data) <- sapply(names(data), function(v){as.character(labeldata[get("variable") == v, "var_label"][1])}, simplify = F)
+      data.table::setkey(labeldata, variable, level)
+      for (i in seq_along(res$CatTable)){
+        for(j in factor_vars){
+          lvs <- res$CatTable[[i]][[j]]$level
+          res$CatTable[[i]][[j]]$level <- labeldata[.(j, lvs), val_label]
+        }
+      }
+      #vals.tb1 <- c(NA, unlist(sapply(vars, function(v){labeldata[get("variable") == v, "val_label"]})))
+    }
+    
     ptb1 <- print(res,
                  showAllLevels = showAllLevels, printToggle = printToggle, quote = quote, smd = smd, varLabels = Labels, nonnormal = nonnormal,
                  catDigits = catDigits, contDigits = contDigits, pDigits = pDigits)
     rownames(ptb1) <- gsub("(mean (SD))", "", rownames(ptb1), fixed=T)
-    cap.tb1 <- "Table 1: Total"
-    if (Labels & !is.null(labeldata)){
-      ptb1[,1] <- vals.tb1
-    }
+    cap.tb1 <- "Total"
+    #if (Labels & !is.null(labeldata)){
+    #  ptb1[,1] <- vals.tb1
+    #}
     return(list(table = ptb1, caption = cap.tb1))
   } else if (is.null(strata2)){
     ptb1 <- CreateTableOne2(strata = strata, vars =vars, data = data, factorVars = factorVars, includeNA = includeNA, test = test, 
@@ -183,31 +220,52 @@ CreateTableOneJS <- function(vars, strata = NULL, strata2 = NULL, data, factorVa
                            showAllLevels = showAllLevels, printToggle = printToggle, quote = quote, Labels = Labels, nonnormal = nonnormal, exact = exact,
                            catDigits = catDigits, contDigits = contDigits, pDigits = pDigits, labeldata = labeldata)
     
-    cap.tb1 <- paste("Table 1: Stratified by ", strata, sep="")
+    cap.tb1 <- paste("Stratified by ", strata, sep="")
     
     if (Labels & !is.null(labeldata)){
-      cap.tb1 <- paste("Table 1: Stratified by ", labeldata[get("variable") == strata, "var_label"][1], sep="")
+      cap.tb1 <- paste("Stratified by ", labeldata[get("variable") == strata, "var_label"][1], sep="")
       #ptb1[,1] = vals.tb1
       
     }
     return(list(table = ptb1, caption = cap.tb1))
   } else if (psub ==T){
-    data.strata <-  lapply(levels(data[[strata]]), function(x){data[data[[strata]] == x, ]})
+    #data.strata <-  lapply(levels(data[[strata]]), function(x){data[data[[strata]] == x, ]})
+    data.strata <- split(data, data[[strata]])
     ptb1.list <- lapply(data.strata, CreateTableOne2,
                        vars =vars, strata = strata2, factorVars = factorVars, includeNA = includeNA, test = test, 
                        testApprox = testApprox, argsApprox = argsApprox,
                        testExact = testExact, argsExact = argsExact,
                        testNormal = testNormal, argsNormal = argsNormal,
                        testNonNormal = testNonNormal, argsNonNormal = argsNonNormal, smd = smd,
-                       showAllLevels = showAllLevels, printToggle = printToggle, quote = quote, Labels = Labels, nonnormal = nonnormal, exact = exact,
-                       catDigits = catDigits, contDigits = contDigits, pDigits = pDigits, labeldata = labeldata
-    )
-    ptb1.cbind = Reduce(cbind, c(list(ptb1.list[[1]]), lapply(2:length(ptb1.list), function(x){ptb1.list[[x]][,-1]})))
+                       showAllLevels = showAllLevels, printToggle = printToggle, quote = quote, Labels = F, nonnormal = nonnormal, exact = exact,
+                       catDigits = catDigits, contDigits = contDigits, pDigits = pDigits)
+    
+    if (showAllLevels == T){
+      ptb1.cbind <- Reduce(cbind, c(list(ptb1.list[[1]]), lapply(2:length(ptb1.list), function(x){ptb1.list[[x]][,-1]})))
+    } else{
+      ptb1.cbind <- Reduce(cbind, ptb1.list)
+    }
+    
     #colnum.test = which(colnames(ptb1.cbind) == "test")
     #ptb1.2group = ptb1.cbind[, c(setdiff(1:ncol(ptb1.cbind), colnum.test), colnum.test[1])]
-    cap.tb1 <- paste("Table 1: Stratified by ", strata, "(", paste(levels(data[[strata]]), collapse=", "), ") & ", strata2, sep="")
+    cap.tb1 <- paste("Stratified by ", strata, "(", paste(levels(data[[strata]]), collapse=", "), ") & ", strata2, sep="")
     if (Labels & !is.null(labeldata)){
-      cap.tb1 <- paste("Table 1: Stratified by ", labeldata[get("variable") == strata, "var_label"][1], "(", paste(unlist(labeldata[get("variable") == strata, "val_label"]), collapse=", "), ") & ", labeldata[get("variable") == strata2, "var_label"][1], sep="")
+      labelled::var_label(data.strata[[1]]) = sapply(names(data.strata[[1]]), function(v){as.character(labeldata[get("variable") == v, "var_label"][1])}, simplify = F)
+      #vals.tb1 <- c(NA, unlist(sapply(vars, function(v){labeldata[get("variable") == v, "val_label"]})))
+      data.table::setkey(labeldata, variable, level)
+      
+      res <- tableone::CreateTableOne(vars =vars, data = data.strata[[1]], factorVars = factorVars, includeNA = includeNA)
+      factor_vars <- res[["MetaData"]][["varFactors"]]
+      for (i in seq_along(res$CatTable)){
+        for(j in factor_vars){
+          lvs <- res$CatTable[[i]][[j]]$level
+          res$CatTable[[i]][[j]]$level <- labeldata[.(j, lvs), val_label]
+        }
+      }
+      ptb1.rn <- rownames(print(res, showAllLevels = showAllLevels, printToggle = printToggle, quote = quote, varLabels = Labels, nonnormal = nonnormal,
+                                catDigits = catDigits, contDigits = contDigits))
+      rownames(ptb1.cbind) <- gsub("(mean (SD))", "", ptb1.rn, fixed=T)
+      cap.tb1 <- paste("Stratified by ", labeldata[get("variable") == strata, "var_label"][1], "(", paste(unlist(labeldata[get("variable") == strata, "val_label"]), collapse=", "), ") & ", labeldata[get("variable") == strata2, "var_label"][1], sep="")
     }
     
     return(list(table = ptb1.cbind, caption = cap.tb1))
@@ -218,7 +276,8 @@ CreateTableOneJS <- function(vars, strata = NULL, strata2 = NULL, data, factorVa
                         testNormal = oneway.test, argsNormal = list(var.equal = F),
                         testNonNormal = kruskal.test, argsNonNormal = list(NULL)) 
     
-    factor_vars <- vars[sapply(vars, function(x){class(data[[x]]) %in% c("factor", "character")})]
+    factor_vars <- res[["MetaData"]][["varFactors"]]
+    #factor_vars <- vars[sapply(vars, function(x){class(data[[x]]) %in% c("factor", "character")})]
     var.strata <- paste(data[[strata2]], data[[strata]], sep = "_")
   
     vars.fisher <- sapply(factor_vars, function(x){is(tryCatch(chisq.test(table(var.strata, data[[x]])),error=function(e) e, warning=function(w) w), "warning")})
@@ -228,12 +287,33 @@ CreateTableOneJS <- function(vars, strata = NULL, strata2 = NULL, data, factorVa
       exact <- vars.fisher
     }
     
+    if (Labels & !is.null(labeldata)){
+      labelled::var_label(data) <- sapply(names(data), function(v){as.character(labeldata[get("variable") == v, "var_label"][1])}, simplify = F)
+      data.table::setkey(labeldata, variable, level)
+      res0 <- tableone::CreateTableOne(vars =vars, data = data, factorVars = factorVars, includeNA = includeNA)
+      for (i in seq_along(res0$CatTable)){
+        for(j in factor_vars){
+          lvs <- res0$CatTable[[i]][[j]]$level
+          res0$CatTable[[i]][[j]]$level <- labeldata[.(j, lvs), val_label]
+        }
+      }
+      ptb1.rn <- rownames(print(res0, showAllLevels = showAllLevels, printToggle = printToggle, quote = quote, varLabels = Labels, nonnormal = nonnormal,
+                                catDigits = catDigits, contDigits = contDigits))
+      ptb1.rn <- gsub("(mean (SD))", "", ptb1.rn, fixed=T)
+      #vals.tb1 <- c(NA, unlist(sapply(vars, function(v){labeldata[get("variable") == v, "val_label"]})))
+    }
+    
     ptb1 <- print(res, 
-                showAllLevels=T,
-                printToggle=F, quote=F, smd = smd, varLabels = T, exact = exact, nonnormal = nonnormal,
+                showAllLevels=showAllLevels,
+                printToggle=F, quote=F, smd = smd, varLabels = Labels, exact = exact, nonnormal = nonnormal,
                 catDigits = catDigits, contDigits = contDigits, pDigits = pDigits)
     
     rownames(ptb1) <- gsub("(mean (SD))", "", rownames(ptb1), fixed=T)
+    if (Labels & !is.null(labeldata)){
+      rownames(ptb1) <- ptb1.rn
+    
+      }
+    
     sig <- ifelse(ptb1[,"p"] == "<0.001", "0", ptb1[,"p"])
     sig <- as.numeric(as.vector(sig))
     sig <- ifelse(sig <= 0.05, "**", "")
@@ -245,12 +325,17 @@ CreateTableOneJS <- function(vars, strata = NULL, strata2 = NULL, data, factorVa
       val_combination <- data.table::CJ(labeldata[variable == strata, val_label], labeldata[variable == strata2, val_label])
       colname.group_var <- val_combination[, paste(V1, ":", V2, sep="")] 
       colname.group_index <- paste(labeldata[variable == strata, var_label][1], ":", labeldata[variable == strata2, var_label][1], sep = "")
-      colnames(ptb1)[1:(length(colname.group_var)+1)] = c(colname.group_index, colname.group_var)
+      if (showAllLevels == T){
+        colnames(ptb1)[1:(length(colname.group_var)+1)] <- c(colname.group_index, colname.group_var)
+      } else{
+        colnames(ptb1)[1:length(colname.group_var)] <- colname.group_var
+      }
+      
       # caption
-      cap.tb1 = paste("Table 1: Stratified by ", labeldata[variable == strata, var_label][1], " and ", labeldata[variable == strata2, var_label][1],  sep="")
+      cap.tb1 = paste("Stratified by ", labeldata[variable == strata, var_label][1], " and ", labeldata[variable == strata2, var_label][1],  sep="")
       # val_label
-      vals.tb1 <- c(NA, unlist(sapply(vars, function(v){labeldata[variable == v, val_label]})))
-      ptb1[,1] <- vals.tb1
+      #vals.tb1 <- c(NA, unlist(sapply(vars, function(v){labeldata[variable == v, val_label]})))
+      #ptb1[,1] <- vals.tb1
     }
     return(list(table = ptb1, caption = cap.tb1))
     
